@@ -15,25 +15,33 @@
 """Base definitions useful across the project."""
 
 import abc
+import dataclasses
 import itertools
-import warnings
 
 from typing import Any
 from typing import Callable
 from typing import NamedTuple
 from typing import Optional
 from typing import Union
+from typing import Tuple
 
 import jax
 import jax.numpy as jnp
 
 from jaxopt import implicit_diff as idf
-from jaxopt import linear_solve
+
+# Do not use jaxopt.linear_solve to avoid circular imports: use
+# jaxopt._src.linear_solve instead.
+# This allows to define linear solver with base.Solver interface,
+# and then exporting them in jaxopt.linear_solve.
+from jaxopt._src import linear_solve
+
 from jaxopt import loop
 from jaxopt import tree_util
 
 
 AutoOrBoolean = Union[str, bool]
+ArrayPair = Tuple[jnp.ndarray, jnp.ndarray]
 
 
 class OptStep(NamedTuple):
@@ -56,6 +64,13 @@ class Solver(abc.ABC):
 
   In addition, we will assume that the solver defines a notion of optimality:
     - `pytree = optimality_fun(params, *args, **kwargs)
+
+  Note: subclasses of `Solver` which are also Python dataclasses must disable
+  the generation of an `__eq__` method (`eq=False`) for compatibility with
+  Python 3.7. Not doing so results in the generation of a `__hash__` method
+  that clashes with JAX transformations when applied directly to methods of
+  the class. This restriction will be lifted in the future when support for
+  Python 3.7 is dropped and / or another solution is found.
   """
 
   @abc.abstractmethod
@@ -70,6 +85,12 @@ class Solver(abc.ABC):
     optimality = self.optimality_fun(params, *args, **kwargs)
     return tree_util.tree_l2_norm(optimality)
 
+  def attribute_names(self):
+    return tuple(field.name for field in dataclasses.fields(self))
+
+  def attribute_values(self):
+    return tuple(getattr(self, name) for name in self.attribute_names())
+
 
 class IterativeSolver(Solver):
   """Base class for iterative solvers.
@@ -95,15 +116,6 @@ class IterativeSolver(Solver):
     - `error`
   """
 
-  def init(self,
-           init_params: Any,
-           *args,
-           **kwargs) -> OptStep:
-    warnings.warn("Method 'init' will be removed in v0.2. "
-                  "Use 'init_state' instead.", FutureWarning)
-    state = self.init_state(init_params, *args, **kwargs)
-    return OptStep(params=init_params, state=state)
-
   def _get_loop_options(self):
     """Returns jit and unroll options based on user-provided attributes."""
 
@@ -118,24 +130,23 @@ class IterativeSolver(Solver):
       unroll = not getattr(self, "implicit_diff", True) or not jit
     else:
       unroll = self.unroll
-    
+
     return jit, unroll
 
+  def _cond_fun(self, inputs):
+    _, state = inputs[0]
+    if self.verbose:
+      print(state.error)
+    return state.error > self.tol
+
+  def _body_fun(self, inputs):
+    (params, state), (args, kwargs) = inputs
+    return self.update(params, state, *args, **kwargs), (args, kwargs)
+
   def _run(self,
            init_params: Any,
            *args,
            **kwargs) -> OptStep:
-
-    def cond_fun(pair):
-      _, state = pair
-      if self.verbose:
-        print(state.error)
-      return state.error > self.tol
-
-    def body_fun(pair):
-      params, state = pair
-      return self.update(params, state, *args, **kwargs)
-
     state = self.init_state(init_params, *args, **kwargs)
 
     if self.maxiter == 0:
@@ -145,12 +156,13 @@ class IterativeSolver(Solver):
     # below to have the same output type, which is a requirement of
     # lax.while_loop and lax.scan.
     opt_step = self.update(init_params, state, *args, **kwargs)
+    init_val = (opt_step, (args, kwargs))
 
     jit, unroll = self._get_loop_options()
 
-    return loop.while_loop(cond_fun=cond_fun, body_fun=body_fun,
-                           init_val=opt_step, maxiter=self.maxiter - 1, jit=jit,
-                           unroll=unroll)
+    return loop.while_loop(cond_fun=self._cond_fun, body_fun=self._body_fun,
+                           init_val=init_val, maxiter=self.maxiter - 1, jit=jit,
+                           unroll=unroll)[0]
 
   def run(self,
           init_params: Any,
