@@ -19,6 +19,7 @@ from typing import Callable
 from typing import Optional
 from typing import Tuple
 
+import warnings
 from dataclasses import dataclass
 
 import jax
@@ -108,7 +109,7 @@ def _solve_constrained_qp_cvxpy(params_obj, params_eq, params_ineq):
   objective = 0.5 * cp.quad_form(x, Q) + c.T @ x
   constraints = [A @ x == b, G @ x <= h]
   pb = cp.Problem(cp.Minimize(objective), constraints)
-  pb.solve()
+  pb.solve(solver='OSQP')
 
   if pb.status in ["infeasible", "unbounded"]:
     raise ValueError("The problem is %s." % pb.status)
@@ -155,9 +156,12 @@ def _make_quadratic_prog_optimality_fun(matvec_Q, matvec_A):
   return idf.make_kkt_optimality_fun(obj_fun, eq_fun, ineq_fun)
 
 
-@dataclass
+@dataclass(eq=False)
 class QuadraticProgramming(base.Solver):
-  """Quadratic programming solver.
+  """Deprecated: will be removed in v0.3.
+
+  Use :class:`jaxopt.CvxpyQP`, :class:`jaxopt.OSQP`, :class:`jaxopt.BoxOSQP` and
+  :class:`jaxopt.EqualityConstrainedQP` instead.
 
   The objective function is::
 
@@ -229,6 +233,14 @@ class QuadraticProgramming(base.Solver):
     return tree_util.tree_l2_norm(pytree)
 
   def __post_init__(self):
+    warnings.warn("Class 'QuadraticProgramming' will be removed in v0.3. "
+                  "Use 'EqualityConstraintsQP' if you want the same behavior as "
+                  "'QuadraticProgramming' for QPs with equality constraints only. "
+                  "Use 'CVXPY_QP' if you want the same behavior as "
+                  "'QuadraticProgramming' for QPs with inequality constraints. "
+                  "Use 'OSQP' if you want a solver that supports pytrees, matvec, jit and vmap "
+                  "for QPs with inequality constraints.", FutureWarning)
+
     self.optimality_fun = _make_quadratic_prog_optimality_fun(self.matvec_Q,
                                                               self.matvec_A)
 
